@@ -14,7 +14,13 @@ import subprocess
 import sys
 from KeyisBClient import Url
 import asyncio
+from io import TextIOWrapper
+import subprocess
 
+def restart_as_root():
+    if os.geteuid() != 0:
+        os.execvp("sudo", ["sudo", sys.executable] + sys.argv)
+restart_as_root()
 
 def _kill_process_by_port(port: int):
 
@@ -124,6 +130,29 @@ def _kill_process_by_port(port: int):
 def _sign(k:bytes)->bytes:nonce=os.urandom(16);m=b"keyisb-vm-host-"+os.urandom(32);return nonce+Cipher(algorithms.ChaCha20(k[:32],nonce),None).encryptor().update(m)
 def _verify(k:bytes,s:bytes)->bool:nonce,ct=s[:16],s[16:];return Cipher(algorithms.ChaCha20(k[:32],nonce),None).decryptor().update(ct).startswith(b"keyisb-vm-host-")
 
+
+
+
+import socket
+
+def is_port_in_use(port: int, proto: str = "tcp") -> bool:
+    if proto == "tcp":
+        sock_type = socket.SOCK_STREAM
+    elif proto == "udp":
+        sock_type = socket.SOCK_DGRAM
+    else:
+        raise ValueError("proto должен быть 'tcp' или 'udp'")
+    
+    s = socket.socket(socket.AF_INET, sock_type)
+    try:
+        s.bind(("0.0.0.0", port))
+    except OSError:
+        return True
+    finally:
+        s.close()
+    return False
+
+
 class App():
     def __init__(self):
         self._app = _App()
@@ -133,6 +162,8 @@ class App():
         self._access_key: Optional[str] = None
 
         self._default_venv_path = None
+        self._default_TLS_paths = None
+        self._log_dir_path: Optional[str] = None
 
         self._client = AsyncClient()
 
@@ -146,22 +177,102 @@ class App():
     def setVenvPath(self, venv_path: str):
         self._default_venv_path = venv_path
 
-    def addServerStartFile(self, name: str, file_path: str, port: Optional[int] = None, start_when_run: bool = False, venv_path: Optional[str] = None):
-        self._servers_start_files[name] = {"name": name, "path": file_path, "port": port, "start_when_run": start_when_run, "venv_path": venv_path if venv_path is not None else self._default_venv_path}
+    def setTLSPaths(self, cert_path: str, key_path: str) -> None:
+        self._default_TLS_paths = (cert_path, key_path)
+
+    def setLogDir(self, dir_path: str) -> None:
+        if os.path.exists(dir_path):
+            self._log_dir_path = dir_path
+        else:
+            raise Exception('Путь не найден')
+
+    def writeLog(self, domain: str, msg: str):
+        file: Optional[TextIOWrapper] = self._servers_start_files[domain].get('_log_file')
+        if file is not None:
+            file.write(msg)
+
+
+    def addServerStartFile(self,
+                           domain: str,
+                           file_path: str,
+                           port: Optional[int] = None,
+                           start_when_run: bool = False,
+                           venv_path: Optional[str] = None,
+                           vm_host: bool = False,
+                           cert_path: Optional[str] = None,
+                           key_path: Optional[str] = None,
+                           host: str = '0.0.0.0',
+                           log_path: Optional[str] = None
+                           ):
+        
+        if log_path is None and self._log_dir_path is not None:
+            log_path = self._log_dir_path + f'/{domain}' + f'{datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")}.log'
+
+        if log_path and not os.path.exists(log_path):
+            os.makedirs(os.path.dirname(log_path), exist_ok=True)
+
+
+
+        self._servers_start_files[domain] = {
+            "domain": domain,
+            "path": file_path,
+            "port": port,
+            "start_when_run": start_when_run,
+            "venv_path": venv_path if venv_path is not None else self._default_venv_path,
+            "vm_host": vm_host,
+            "cert_path": cert_path if cert_path is not None else self._default_TLS_paths[0] if self._default_TLS_paths is not None else None,
+            "key_path": key_path if key_path is not None else self._default_TLS_paths[1] if self._default_TLS_paths is not None else None,
+            "host": host,
+            "log_path": log_path
+            }
+
+        
+        if log_path is not None:
+            file = open(log_path, 'w+')
+            
+            self._servers_start_files[domain]["_log_file"] = file
+
+
 
     async def startLikeRun(self):
         for server in self._servers_start_files:
             if self._servers_start_files[server]["start_when_run"]:
                 asyncio.create_task(self.startServer(server))
 
-    async def startServer(self, name: str, timeout: float = 10.0) -> bool:
-        if name in self._servers_start_files:
-            server = self._servers_start_files[name]
+    async def startServer(self, domain: str, timeout: float = 30):
+        if domain in self._servers_start_files:
+
+
+            res = await self.checkServerHealth(domain, timeout=1.0)
+            if res[0]:
+                raise ValueError(f"Server alredy running: {domain}")
+            
+
+
+            server = self._servers_start_files[domain]
             path = server["path"]
+            port = server["port"]
             venv_path = server["venv_path"]
+            vm_host = server['vm_host']
+            host = server['host']
+            
+            cert_path = server['cert_path']
+            key_path = server['key_path']
+
+            if cert_path is None or key_path is None:
+                raise Exception('Не указаны сертификаты')
+            
             if not os.path.isfile(path):
                 raise ValueError(f"Server start file not found: {path}")
             
+            
+            if is_port_in_use(port, 'udp'):
+                _kill_process_by_port(port)
+                await asyncio.sleep(1)
+
+
+            out_ = None
+
             if path.endswith('.py'):
                 if venv_path is not None:
                     if not os.path.isdir(venv_path):
@@ -171,65 +282,142 @@ class App():
                         raise ValueError(f"Python executable not found in virtual environment: {python_executable}")
                 else:
                     python_executable = sys.executable
+
+                if not vm_host:
+                    argv = []
+                else:
+                    argv = [
+                        'gn:vm-host:start',
+                        domain,
+                        port,
+                        cert_path,
+                        key_path,
+                        host
+                    ]
+
+                self.writeLog(domain, 'Server starting...')
+                proc = subprocess.Popen(
+                    [python_executable, path, *argv],
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    text=True
+                )
+
+                save_to_out = True
+                save_out = []
+
+                async def log_stream(stream, prefix):
+                    while True:
+                        line = await asyncio.to_thread(stream.readline)
+                        if not line:
+                            break
+
+                        if save_to_out:
+                            save_out.append(line.strip())
+
+                        self.writeLog(domain, f"{prefix}: {line.strip()}")
+
+                asyncio.create_task(log_stream(proc.stdout, "STDOUT"))
+                asyncio.create_task(log_stream(proc.stderr, "STDERR"))
+
+                try:
+                    returncode = await asyncio.to_thread(proc.wait, 3)
+                    self.writeLog(domain, f"Process exited with code {returncode}")
+                    save_out.clear()
+                except subprocess.TimeoutExpired:
+                    self.writeLog(domain, "Process timed out")
+                    out_ = '\n'.join(save_out)
+
+                save_to_out = False
+
+                # try:
+                #     returncode = proc.wait(timeout=3)
+                #     out, err = proc.communicate()
+                #     self.writeLog(domain, f'Server error: {returncode}')
+                #     self.writeLog(domain, f'stdout: {out}')
+                #     self.writeLog(domain, f'stderr: {err}')
+
+                #     print("Процесс завершился с кодом:", returncode)
+                #     print("STDOUT:", out)
+                #     print("STDERR:", err)
+                #     out_ = out + '\n\n' + err
+                # except subprocess.TimeoutExpired:
+                #     pass
                 
-                subprocess.Popen([python_executable, path], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
             else:
+                self.writeLog(domain, 'Server starting...')
                 subprocess.Popen([path], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
             
-            return await self.checkServerHealth(name, timeout=timeout)
+            r =  await self.checkServerHealth(domain, timeout=timeout, out=out_)
+            self.writeLog(domain, f'Server checkServerHealth status-> {r[0]}')
+            return r
         else:
-            raise ValueError(f"No server start file found with name: {name}")
+            self.writeLog(domain, f'No server start file found with domain: {domain}')
+            raise ValueError(f"No server start file found with domain: {domain}")
     
 
 
-    async def _send_message_to_server(self, name: str, path: str, payload: Optional[dict] = None, timeout: float = 1.0) -> Optional[GNResponse]:
-        port = self._servers_start_files[name].get("port")
+    async def _send_message_to_server(self, domain: str, path: str, payload: Optional[dict] = None, timeout: float = 1.0, res: list = None) -> Optional[GNResponse]:
+        port = self._servers_start_files[domain].get("port")
         if port is None:
-            raise ValueError(f"No port specified for server: {name}")
+            raise ValueError(f"No port specified for server: {domain}")
 
         if path.startswith('/'):
             path = path[1:]
 
-        c = self._client.request(GNRequest('POST', Url(f'gn://127.0.0.1:{port}/!gn-vm-host/{path}'), payload=payload))
+        c = self._client.request(GNRequest('POST', Url(f'gn://127.0.0.1:{port}/!gn-vm-host/{path}'), payload=payload), reconnect_wait=2)
+        
+
+        print(f'send request timeout: {timeout} (port: {port})')
 
         try:
             result = await asyncio.wait_for(c, timeout=timeout)
         except asyncio.TimeoutError:
             result = None
         
-        return result
+        print(f'result -> {result}')
+        
+        res.append(result)
 
-    async def checkServerHealth(self, name: str, timeout: float = 3.0, interval=0.5):
-        end = asyncio.get_event_loop().time() + timeout
-        while asyncio.get_event_loop().time() < end:
-            result = await self._send_message_to_server(name, '/ping', timeout=timeout)
-            if result is not None:
-                return True
+    async def checkServerHealth(self, domain: str, timeout: float = 3.0, interval=5, out: Optional[str] = None):
+        loop = asyncio.get_event_loop()
+        end = loop.time() + timeout
+        res = []
+        count = 0
+        while loop.time() < end:
+            loop.create_task(self._send_message_to_server(domain, '/ping', timeout=timeout - interval * count, res=res))
+            count+=1
+
+            await asyncio.sleep(0.05)
+
+            if res != []:
+                return (True, out, res[0])
             else:
                 await asyncio.sleep(interval)
-        return False
+            
+        return (False, out, None)
         
 
 
-    def stopServer(self, name: str):
-        if name in self._servers_start_files:
-            server = self._servers_start_files[name]
+    def stopServer(self, domain: str):
+        if domain in self._servers_start_files:
+            server = self._servers_start_files[domain]
             port = server["port"]
             if port is not None:
                 _kill_process_by_port(port)
             else:
-                raise ValueError(f"No port specified for server: {name}")
+                raise ValueError(f"No port specified for server: {domain}")
         else:
-            raise ValueError(f"No server start file found with name: {name}")
+            raise ValueError(f"No server start file found with domain: {domain}")
 
-    async def reloadServer(self, name: str, timeout: float = 1):
-        if name in self._servers_start_files:
-            self.stopServer(name)
+    async def reloadServer(self, domain: str, timeout: float = 1):
+        if domain in self._servers_start_files:
+            self.stopServer(domain)
             await asyncio.sleep(timeout)
-            return await self.startServer(name)
+            return await self.startServer(domain)
         else:
-            raise ValueError(f"No server start file found with name: {name}")
+            raise ValueError(f"No server start file found with domain: {domain}")
 
     def run(self,
             host,
@@ -242,15 +430,14 @@ class App():
             ):
         
 
-        asyncio.create_task(self.startLikeRun())
-
         self._app.run(
             host=host,
             port=port,
             cert_path=cert_path,
             key_path=key_path,
             idle_timeout=idle_timeout,
-            wait=wait
+            wait=wait,
+            run=self.startLikeRun
         )
 
 
@@ -267,19 +454,19 @@ class App():
 
     def __add_routes(self):
         @self._app.route('POST', '/ping')
-        async def ping_handler(request: GNRequest, name: Optional[str] = None, timeout: float = 3.0):
+        async def ping_handler(request: GNRequest, domain: Optional[str] = None, timeout: float = 3.0):
             if not self.__resolve_access_key(request):
                 return None
             
-            if not name:
+            if not domain:
                 return GNResponse('ok', {'time': datetime.datetime.now(datetime.timezone.utc).isoformat()})
             else:
                 try:
-                    result = await self.checkServerHealth(name, timeout=timeout)
-                    if result:
-                        return GNResponse('ok', {'message': f'Server {name} is alive.'})
+                    result = await self.checkServerHealth(domain, timeout=timeout)
+                    if result[0]:
+                        return GNResponse('ok', {'message': f'Server {domain} is alive.', 'time': result[2].payload['time']})
                     else:
-                        return GNResponse('error', {'error': f'Server {name} is not responding.'})
+                        return GNResponse('error', {'error': f'Server {domain} is not responding.'})
                 except ValueError as e:
                     return GNResponse('error', {'error': str(e)})
                 
@@ -290,11 +477,7 @@ class App():
             
             servers_info = []
             for server in self._servers_start_files.values():
-                servers_info.append({
-                    'name': server['name'],
-                    'port': server['port'],
-                    'start_when_run': server['start_when_run']
-                })
+                servers_info.append(server['domain'])
             return GNResponse('ok', {'servers': servers_info})
 
 
@@ -302,51 +485,51 @@ class App():
             
 
         @self._app.route('POST', '/start-server')
-        async def start_server_handler(request: GNRequest, name: str = ''):
+        async def start_server_handler(request: GNRequest, domain: str = ''):
             if not self.__resolve_access_key(request):
                 return None
             
-            if not name:
-                return GNResponse('error', {'error': 'Server name is required.'})
+            if not domain:
+                return GNResponse('error', {'error': 'Server domain is required.'})
             try:
-                result = await self.startServer(name)
-                if result:
-                    return GNResponse('ok', {'message': f'Server {name} started.'})
+                result = await self.startServer(domain)
+                if result[0]:
+                    return GNResponse('ok', {'message': f'Server {domain} started.'})
                 else:
-                    return GNResponse('error', {'error': f'Server {name} failed to start within the timeout period.'})
+                    return GNResponse('error', {'error': f'Server {domain} failed to start within the timeout period. {result[1]}'})
             except ValueError as e:
                 return GNResponse('error', {'error': str(e)})
             
             
 
         @self._app.route('POST', '/reload-server')
-        async def reload_server_handler(request: GNRequest, name: str = '', timeout: float = 0.5):
+        async def reload_server_handler(request: GNRequest, domain: str = '', timeout: float = 0.5):
             if not self.__resolve_access_key(request):
                 return None
 
-            if not name:
-                return GNResponse('error', {'error': 'Server name is required.'})
+            if not domain:
+                return GNResponse('error', {'error': 'Server domain is required.'})
 
             try:
-                result = await self.reloadServer(name, timeout)
-                if result:
-                    return GNResponse('ok', {'message': f'Server {name} reloaded.'})
+                result = await self.reloadServer(domain, timeout)
+                if result[0]:
+                    return GNResponse('ok', {'message': f'Server {domain} reloaded.'})
                 else:
-                    return GNResponse('error', {'error': f'Server {name} failed to reload within the timeout period.'})
+                    return GNResponse('error', {'error': f'Server {domain} failed to reload within the timeout period. {result[1]}'})
             except ValueError as e:
                 return GNResponse('error', {'error': str(e)})
         
         @self._app.route('POST', '/stop-server')
-        async def stop_server_handler(request: GNRequest, name: str = ''):
+        async def stop_server_handler(request: GNRequest, domain: str = ''):
             if not self.__resolve_access_key(request):
                 return None
 
-            if not name:
-                return GNResponse('error', {'error': 'Server name is required.'})
+            if not domain:
+                return GNResponse('error', {'error': 'Server domain is required.'})
 
             try:
-                self.stopServer(name)
-                return GNResponse('ok', {'message': f'Server {name} stopped.'})
+                self.stopServer(domain)
+                return GNResponse('ok', {'message': f'Server {domain} stopped.'})
             except ValueError as e:
                 return GNResponse('error', {'error': str(e)})
         
